""" method to obtain data from remote Apache Airflow server """

from typing import List

import airflow_client.client
import yaml
from airflow_client.client.api import connection_api, pool_api, variable_api

from airflow_helper.api import AirflowAPIBase
from airflow_helper.models import AirflowConfig, Config, Connection, Pool, Variable
from airflow_helper.settings import logger
from airflow_helper.settings import settings as s


class RemoteConfigObtainter:
    def __init__(self, by_api: bool = True, **kwargs) -> None:
        if by_api:
            self.connector = AirflowAPIGrabber(**kwargs)
        else:
            self.connector = DBGrabber(**kwargs)

    def dump_config(self, file_path: str = s.config_file_name) -> None:
        config = self.connector.collect_full_config()

        logger.info(f"Dumping config to the path {file_path}")

        with open(file_path, "w+") as f:
            yaml.dump(config.model_dump(exclude_unset=True), f)


class DBGrabber:
    def __init__(self):
        raise NotImplementedError(
            "Getting Variables, Connections & Pools directly from Airflow DB will be available in next releases"
        )


class AirflowAPIGrabber(AirflowAPIBase):
    def get_pools(self) -> List[Pool]:
        with airflow_client.client.ApiClient(self.conn_config) as api_client:
            pools = pool_api.PoolApi(api_client).get_pools()["pools"]
            pools_list = []
            for pool in pools:
                pools_list.append(Pool(**pool._data_store))
            return pools_list

    def get_variables(self) -> List[Variable]:
        with airflow_client.client.ApiClient(self.conn_config) as api_client:
            variables = variable_api.VariableApi(api_client).get_variables()[
                "variables"
            ]
            variables_list = []
            for variable in variables:
                variables_list.append(Variable(**variable._data_store))
            return variables_list

    def get_connections(self) -> List[Connection]:
        with airflow_client.client.ApiClient(self.conn_config) as api_client:
            connections = connection_api.ConnectionApi(api_client).get_connections()[
                "connections"
            ]
            connections_list = []
            for connection in connections:
                connections_list.append(Connection(**connection._data_store))
            return connections_list

    def collect_full_config(self) -> AirflowConfig:
        return Config(
            airflow=AirflowConfig(
                variables=self.get_variables(),
                connections=self.get_connections(),
                pools=self.get_pools(),
            )
        )



import yaml

from airflow_helper.models import Config, ConfigFile
from airflow_helper.settings import settings as s


class ConfigReader:
    def __init__(self, file_path: str = s.config_file_name) -> None:
        self.config_file = ConfigFile(file_path=file_path)
        self.config = None

    @staticmethod
    def _populate_config_data(config_data: dict, added_config: dict) -> None:
        # we adding connections, pools, variables, not overwrite
        for key, value in added_config.get("airflow", {}).items():
            if not config_data["airflow"].get(key):
                config_data["airflow"][key] = []
            config_data["airflow"][key].extend(value)

    def resolve_included_config_path(self, path: str) -> str:
        if "/" not in path:
            path = self.config_file.file_path.parents[0] / path
        return path

    def process_included_configs(self, base_config_data: dict) -> None:
        include_configs = base_config_data.get("include", [])
        config_data = {"airflow": {}}
        for path in include_configs:
            if path.strip():
                included_config_path = self.resolve_included_config_path(path)
                parent_config = self.read_config_yaml(included_config_path)
                if parent_config:
                    self._populate_config_data(config_data, parent_config)
                    if parent_config.get("include"):
                        parent_config_included_data = self.process_included_configs(
                            parent_config
                        )
                        self._populate_config_data(
                            config_data, parent_config_included_data
                        )
        self._populate_config_data(config_data, base_config_data)
        return config_data

    def load_config(self):
        base_config_data = self.read_config_yaml(self.config_file.file_path)
        config_data = self.process_included_configs(base_config_data)
        self.config = Config(**config_data)
        return self.config

    def read_config_yaml(self, file_path: str):
        with open(file_path, "r") as f:
            return yaml.safe_load(f)



import yaml

from airflow_helper.models import EmptyConfig
from airflow_helper.settings import settings as s


class EmptyConfigCreator:
    def __init__(self, file_path: str = s.config_file_name) -> None:
        self.file_path = file_path
        self.config = None

    def create(self):
        with open(self.file_path, "w+") as f:
            yaml.dump(EmptyConfig().model_dump(), f)



from typing import Optional

import airflow_client.client
from pydantic import validate_call

from airflow_helper.settings import logger
from airflow_helper.settings import settings as s


class AirflowAPIBase:
    @validate_call
    def __init__(
        self,
        user: Optional[str] = None,
        password: Optional[str] = None,
        url: Optional[str] = None,
        port: Optional[int] = None,
        host: Optional[str] = None,
    ) -> None:
        self.url = self.get_airflow_url(url=url, port=port, host=host)
        self.conn_config = airflow_client.client.Configuration(
            host=self.url,
            username=user or s.user,
            password=password or s.password,
        )

    @staticmethod
    def get_airflow_url(
        url: Optional[str] = None,
        port: Optional[int] = None,
        host: Optional[str] = None,
    ):
        if url:
            logger.info(f"Provided full url to Airflow Server, will be used: {url}")
            if url.endswith("/"):
                url = url[:-1]
        else:
            if not port:
                logger.info(f"Port is not provided. Will be used default: {s.port}")
                port = s.port
            if not host:
                logger.info(f"Host is not provided. Will be used default: {s.host}")
                host = s.host
            url = f"{host}:{port}"
        if "://" not in url:
            logger.info(
                f"Protocol is not provided in the host url. Will be used default: {s.protocol}"
            )
            url = f"{s.protocol}{url}"
        return f"{url}/{s.api_postfix}"



from typing import Callable, List, Union

import airflow_client
from airflow_client.client.api import connection_api, pool_api, variable_api
from airflow_client.client.api.connection_api import Connection as APIConnection
from airflow_client.client.api.pool_api import Pool as APIPool
from airflow_client.client.api.variable_api import Variable as APIVariable

from airflow_helper.api import AirflowAPIBase
from airflow_helper.models import Connection, Pool, Variable
from airflow_helper.reader import ConfigReader
from airflow_helper.settings import logger
from airflow_helper.settings import settings as s


class ConfigUploader:
    def __init__(
        self,
        file_path: str = s.config_file_name,
        overwrite: bool = s.overwrite,
        **kwargs,
    ) -> None:
        self.api = PostAirflowAPI(overwrite=overwrite, **kwargs)
        self.overwrite = overwrite
        self.config = ConfigReader(file_path=file_path).load_config()

    def upload_config_to_server(self):
        logger.info(f"Uploading connections to airflow server: {self.api.url}")
        self.api.create_connections(self.config.airflow.connections)
        logger.info(f"Uploading pools to airflow server: {self.api.url}")
        self.api.create_pools(self.config.airflow.pools)
        logger.info(f"Uploading variables to airflow server: {self.api.url}")
        self.api.create_variables(self.config.airflow.variables)


class PostAirflowAPI(AirflowAPIBase):
    def __init__(self, overwrite: bool = False, **kwargs):
        self.overwrite = overwrite
        super().__init__(**kwargs)

    def post_items(
        self,
        data: List[Union[Connection, Variable, Pool]],
        api_method: Callable,
        api_model: Union[APIConnection, APIPool, APIVariable],
    ):
        items_to_patch = []
        for item in data:
            try:
                api_method(api_model(**item.model_dump(exclude_none=True)))
            except airflow_client.client.exceptions.ApiException as e:
                if "409" in str(e):
                    # mean we have already duplicate
                    if self.overwrite:
                        items_to_patch.append(item)
                else:
                    raise e
        return items_to_patch

    def patch_connections(
        self, items_to_patch: List[Pool], api_client: airflow_client.client.ApiClient
    ) -> None:
        for connection in items_to_patch:
            connection_api.ConnectionApi(api_client).patch_connection(
                connection.connection_id,
                APIConnection(**connection.model_dump(exclude_none=True)),
            )

    def create_connections(self, connections: List[Connection]) -> None:
        with airflow_client.client.ApiClient(self.conn_config) as api_client:
            items_to_patch = self.post_items(
                connections,
                connection_api.ConnectionApi(api_client).post_connection,
                APIConnection,
            )
            if self.overwrite and items_to_patch:
                self.patch_connections(items_to_patch, api_client)

    def patch_pools(
        self, items_to_patch: List[Pool], api_client: airflow_client.client.ApiClient
    ) -> None:
        for pool in items_to_patch:
            pool_api.PoolApi(api_client).patch_pool(
                pool.name, APIPool(**pool.model_dump(exclude_none=True))
            )

    def create_pools(self, pools: List[Connection]) -> None:
        with airflow_client.client.ApiClient(self.conn_config) as api_client:
            items_to_patch = self.post_items(
                pools, pool_api.PoolApi(api_client).post_pool, APIPool
            )
            if self.overwrite and items_to_patch:
                self.patch_pools(items_to_patch, api_client)

    def create_variables(self, variables: List[Variable]) -> None:
        with airflow_client.client.ApiClient(self.conn_config) as api_client:
            self.post_items(
                variables,
                variable_api.VariableApi(api_client).post_variables,
                APIVariable,
            )



from airflow_helper.core import ConfigUploader
from airflow_helper.creator import EmptyConfigCreator
from airflow_helper.reader import ConfigReader
from airflow_helper.remote import RemoteConfigObtainter

__all__ = [
    "RemoteConfigObtainter",
    "ConfigReader",
    "ConfigUploader",
    "EmptyConfigCreator",
]



from typing import Optional

import typer
from typing_extensions import Annotated

from airflow_helper import ConfigUploader, EmptyConfigCreator, RemoteConfigObtainter
from airflow_helper.settings import settings as s

CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])


main_app = typer.Typer(context_settings=CONTEXT_SETTINGS)

create_app = typer.Typer()
server_args = []


@create_app.command(help="Create new empty config")
def new(
    file_path: Annotated[
        Optional[str], typer.Argument(..., help="File path there to store config.")
    ] = s.config_file_name,
):
    EmptyConfigCreator(file_path=file_path).create()


@create_app.command(help="Create config with values from existed Airflow Server")
def from_server(
    file_path: Annotated[
        Optional[str], typer.Argument(..., help="File path where to store config.")
    ] = s.config_file_name,
    url: Annotated[
        str,
        typer.Option(
            help="Apache Airflow full url to connect. You can provide it or host & port separately."
        ),
    ] = None,
    host: Annotated[
        str,
        typer.Option(
            help="Apache Airflow server host form that obtain existed settings"
        ),
    ] = s.host,
    port: Annotated[
        str,
        typer.Option(
            help="Apache Airflow server port form that obtain existed settings"
        ),
    ] = s.port,
    user: Annotated[
        str, typer.Option("--user", "-u", help="Apache Airflow user with read rights")
    ] = s.user,
    password: Annotated[
        str, typer.Option("--password", "-p", help="Apache Airflow user password")
    ] = s.password,
):
    RemoteConfigObtainter(
        user=user, password=password, url=url, host=host, port=port
    ).dump_config(file_path=file_path)


@main_app.command(help="Load settings from yaml config to the Apache Airflow server")
def load(
    file_path: Annotated[
        Optional[str], typer.Argument(..., help="File path to config.")
    ] = s.config_file_name,
    url: Annotated[
        str,
        typer.Option(
            help="Apache Airflow full url to connect. You can provide it or host & port separately."
        ),
    ] = None,
    host: Annotated[
        str,
        typer.Option(
            help="Apache Airflow server host form that obtain existed settings"
        ),
    ] = s.host,
    port: Annotated[
        str,
        typer.Option(
            help="Apache Airflow server port form that obtain existed settings"
        ),
    ] = s.port,
    user: Annotated[
        str, typer.Option("--user", "-u", help="Apache Airflow user with read rights")
    ] = s.user,
    password: Annotated[
        str, typer.Option("--password", "-p", help="Apache Airflow user password")
    ] = s.password,
    overwrite: Annotated[
        bool, typer.Option("--overwrite", "-o", help="Overwrite Connections & Pools if they already exists")
    ] = False,
):
    ConfigUploader(
        overwrite=overwrite, file_path=file_path, url=url, host=host, port=port, user=user, password=password
    ).upload_config_to_server()


main_app.add_typer(
    create_app,
    name="create",
    help="Create new airflow settings config - from existed server or empty",
)


def cli():
    main_app()



from typing import List, Optional

from pydantic import BaseModel, ConfigDict, Field, FilePath, constr


class ConfigFile(BaseModel):
    file_path: FilePath


class EmptyConnection(BaseModel):
    connection_id: str = Field(alias="id", default="")
    conn_type: str = Field(alias="type", default="")
    host: Optional[str] = ""
    login: Optional[str] = ""
    password: Optional[str] = ""
    port: Optional[int] = 5000
    extra: Optional[str] = ""

    model_config = ConfigDict(
        populate_by_name=True,
    )


class Connection(EmptyConnection):
    connection_id: constr(min_length=1) = Field(alias="id")
    conn_type: str = Field(alias="type")
    host: Optional[str] = None
    login: Optional[str] = None
    password: Optional[str] = None
    port: Optional[int] = None
    extra: Optional[str] = None


class EmptyPool(BaseModel):
    name: Optional[str] = ""
    slots: Optional[int] = 120
    description: Optional[str] = ""
    include_deferred: bool = False


class Pool(EmptyPool):
    name: constr(min_length=1)
    slots: int
    description: Optional[str] = None


class EmptyVariable(BaseModel):
    key: Optional[str] = ""
    value: Optional[str] = ""
    description: Optional[str] = ""


class Variable(EmptyVariable):
    key: constr(min_length=1)
    value: str


class AirflowConfig(BaseModel):
    connections: Optional[List[Connection]] = None
    pools: Optional[List[Pool]] = None
    variables: Optional[List[Variable]] = None


class EmptyAirflowConfig(BaseModel):
    connections: List[EmptyConnection] = [EmptyConnection()]
    pools: List[EmptyPool] = [EmptyPool()]
    variables: List[EmptyVariable] = [EmptyVariable()]


class EmptyConfig(BaseModel):
    include: List[str] = [""]
    project_name: Optional[str] = ""
    airflow: EmptyAirflowConfig = EmptyAirflowConfig()


class Config(EmptyConfig):
    airflow: AirflowConfig



import logging

from pydantic_settings import BaseSettings, SettingsConfigDict

logger = logging.getLogger(__file__)


class Settings(BaseSettings):
    # remote connection settings
    port: int = 8080
    host: str = "http://localhost"
    protocol: str = "http://"
    api_postfix: str = "api/v1"
    # airflow default pass & login
    password: str = "airflow"
    user: str = "airflow"
    # config
    config_file_name: str = "airflow_settings.yaml"
    # upload settings
    overwrite: bool = False

    model_config = SettingsConfigDict(env_prefix="airflow_helper_")


settings = Settings()
